"""Main entrypoint."""
import argparse
import secrets

import waitress

from flask import Flask
from flask_pyoidc import OIDCAuthentication
from flask_pyoidc.provider_configuration import ClientMetadata, ProviderConfiguration

from frontpage import config, routes
from frontpage.config import OidcConfig

app = Flask(__name__)


def _init_oidc(cfg: OidcConfig) -> OIDCAuthentication:
    if cfg.client_secret_file is not None:
        with open(cfg.client_secret_file, encoding="UTF-8") as f:
            secret = f.read()
    else:
        secret = cfg.client_secret
    client = ClientMetadata(client_id=cfg.client_id, client_secret=secret)

    # Ensure we have the minimum set of scopes for things to work.
    scopes = {"openid"}
    scopes.update(cfg.scopes)

    auth_params = {"scope": list(scopes)}
    provider = ProviderConfiguration(
        issuer=cfg.issuer, client_metadata=client, auth_request_params=auth_params
    )

    auth = OIDCAuthentication({"default": provider}, app)
    return auth


def main():
    """
    Entrypoint.
    """
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "-c", "--config", metavar="FILE", help="configuration file", required=True
    )
    args = parser.parse_args()

    cfg = config.load(args.config)
    auth = _init_oidc(cfg.oidc)

    app.register_blueprint(routes.register(auth, "default"))

    app.config.update({"DEBUG": cfg.core.debug, "SECRET_KEY": secrets.token_hex()})
    app.config["user_config"] = cfg

    if cfg.core.debug:
        # Reloader doesn't work on Nix
        app.run(use_reloader=False)
    else:
        waitress.serve(app, host="127.0.0.1", port=cfg.core.port)
